'use client';

import React, {Dispatch, FC, SetStateAction, useEffect, useState} from 'react';
import { usePathname } from 'next/navigation';
import classNames from 'classnames';
import Link from 'next/link';
import {Modal, Form, Calendar, Radio } from 'antd';
import type { CalendarProps, RadioChangeEvent  } from 'antd';
import { CloseOutlined } from '@ant-design/icons';
import { RegisterContent, ResetContent, FinishContent, EnterContent } from './authModalContent';
import dayjs, { Dayjs } from 'dayjs';
import {ExpertDetails, ExpertScheduler, Tags} from "../../types/experts";
import { createStyles } from 'antd-style';
import {useLocalStorage} from "../../hooks/useLocalStorage";
import {AUTH_TOKEN_KEY} from "../../constants/common";
import {getSchedulerByExpertId, getSchedulerSession} from "../../actions/experts";
import {ElementsForm} from "../stripe/ElementsForm";

type SchedulerModalProps = {
    open: boolean;
    handleCancel: () => void;
    mode: 'data' | 'time' | 'pay' | 'finish';
    updateMode: (mode: 'data' | 'time' | 'pay' | 'finish') => void;
    sessionCost: number;
    expertId: string;
    locale: string;
};

const useStyle = createStyles(({ token, css, cx }) => {
    const lunar = css`
    color: ${token.colorTextTertiary};
    font-size: ${token.fontSizeSM}px;
  `;
    return {
        wrapper: css`
      width: 450px;
      border: 1px solid ${token.colorBorderSecondary};
      border-radius: ${token.borderRadiusOuter};
      padding: 5px;
    `,
        dateCell: css`
      position: relative;
      &:before {
        content: '';
        position: absolute;
        left: 0;
        right: 0;
        top: 0;
        bottom: 0;
        margin: auto;
        max-width: 40px;
        max-height: 40px;
        background: transparent;
        transition: background 300ms;
        border-radius: ${token.borderRadiusOuter}px;
        border: 1px solid transparent;
        box-sizing: border-box;
      }
      &:hover:before {
        background: rgba(0, 0, 0, 0.04);
      }
    `,
        today: css`
      &:before {
        border: 1px solid ${token.colorPrimary};
      }
    `,
        text: css`
      position: relative;
      z-index: 1;
    `,
        lunar,
        current: css`
      color: ${token.colorTextLightSolid};
      &:before {
        background: ${token.colorPrimary};
      }
      &:hover:before {
        background: ${token.colorPrimary};
        opacity: 0.8;
      }
      .${cx(lunar)} {
        color: ${token.colorTextLightSolid};
        opacity: 0.9;
      }
    `,
        monthCell: css`
      width: 120px;
      color: ${token.colorTextBase};
      border-radius: ${token.borderRadiusOuter}px;
      padding: 5px 0;
      &:hover {
        background: rgba(0, 0, 0, 0.04);
      }
    `,
        monthCellCurrent: css`
      color: ${token.colorTextLightSolid};
      background: ${token.colorPrimary};
      &:hover {
        background: ${token.colorPrimary};
        opacity: 0.8;
      }
    `,
        weekend: css`
      color: ${token.colorError};
      &.gray {
        opacity: 0.4;
      }
    `,
    };
});

export const SchedulerModal: FC<SchedulerModalProps> = ({
   open,
   handleCancel,
   mode,
   updateMode,
    sessionCost,
    locale,
    expertId,
}) => {
    const { styles } = useStyle({ test: true });
    const [selectDate, setSelectDate] = React.useState<Dayjs>(dayjs());
    const [dates, setDates] = React.useState<any>();
    const [tags, setTags] = React.useState<Tags[]>([]);
    const [tag, setTag] = React.useState<number>(-1);
    const [slot, setSlot] = React.useState<string>('');
    const [sessionId, setSessionId] = React.useState<number>(-1);
    const [rawScheduler, setRawScheduler] = useState<ExpertScheduler | null>(null);
    const [jwt] = useLocalStorage(AUTH_TOKEN_KEY, '');

    useEffect( ()=> {
        async function loadScheduler(){
            const rawScheduler = await getSchedulerByExpertId(expertId as string, locale as string, jwt)
            setRawScheduler(rawScheduler)
        }
        if (open) {
            loadScheduler()
        }
    }, [open])

    useEffect(() => {
        const map = {} as any
        rawScheduler?.availableSlots.forEach((el)=>{
            const key = dayjs(el.startTime).format('YYYY-MM-DD');
            if (!map[key]){
                map[key] = []
            }
            map[key].push(el)

        })
        console.log(rawScheduler, map)
        setDates(map)
        setTags(rawScheduler?.tags)
    }, [rawScheduler]);

    const onPanelChange = (value: Dayjs, mode: CalendarProps<Dayjs>['mode']) => {
        console.log(value.format('YYYY-MM-DD'), mode);
    };

    const onDateChange: CalendarProps<Dayjs>['onSelect'] = (value, selectInfo) => {
        if (selectInfo.source === 'date') {
            setSelectDate(value);
            updateMode('time')
        }
    };

    const disabledDate = (currentDate: Dayjs) => {
       return !dates || !dates[currentDate.format('YYYY-MM-DD')]
    }

    const handleTimeslot = (e: RadioChangeEvent) => {
        setSlot(e.target.value.startTime)
        console.log('radio checked', e.target.value);
    };

    const handleTag = (e: RadioChangeEvent) => {
        setTag(e.target.value)
        console.log('tag radio checked', e.target.value);
    };

    const handleSingupSession = async () => {
        const data = {coachId: expertId, tagId: tag, startAtUtc: slot, clientComment:''}
        console.log(data)
        const session = await getSchedulerSession({coachId: expertId, tagId: tag, startAtUtc: slot, clientComment:''}, locale, jwt)
        console.log(session);
        // тут должна быть проверка все ли с регистрацией сессии
        setSessionId(session?.sessionId)
        updateMode('pay')
    }

    const currentDay = dayjs()

    const cellRender: CalendarProps<Dayjs>['fullCellRender'] = (date, info) => {
        const isWeekend = date.day() === 6 || date.day() === 0;
        return React.cloneElement(info.originNode, {
            ...info.originNode.props,
            className: classNames(styles.dateCell, {
                [styles.current]: selectDate.isSame(date, 'date'),
                [styles.today]: date.isSame(dayjs(), 'date'),
            }),
            children: (
                <div className={styles.text}>
            <span
                className={classNames({
                    [styles.weekend]: isWeekend,
                })}
            >
              {date.get('date')}
            </span>
                </div>
            ),
        });
    }


    return (
        <Modal
            className="b-modal"
            open={open}
            title={undefined}
            onOk={undefined}
            onCancel={handleCancel}
            footer={false}
            width={498}
            closeIcon={<CloseOutlined style={{ fontSize: 20, color: '#000' }}/>}
        >
            <div>
                {tags && (
                    <Radio.Group name="radiogroupTags" onChange={handleTag}>
                    {tags?.map((tag)=>(
                        <Radio key={tag.id} value={tag.id}>{tag.name}</Radio>
                        )
                    )}
                    </Radio.Group>
                    )
                }
            </div>
            {mode === 'data' && (
                <Calendar
                    fullscreen={false}
                    onPanelChange={onPanelChange}
                    fullCellRender={cellRender}
                    onSelect={onDateChange}
                    disabledDate={disabledDate}
                    headerRender={({ value, type, onChange, onTypeChange }) => {
                        const start = 0;
                        const end = 12;
                        const monthOptions = [];

                        let current = currentDay.clone();
                        const localeData = value.locale();
                        const months = [];

                        for(let i=0; i<6; i++){
                            const m = current.clone()
                            months.push(m);
                            current = current.add(1,'month')
                        }
                        return (<>
                            {months.map((m, i)=>(
                                <button key={'SchedulerMonth'+i} onClick={()=> onChange(m)}>
                                    {m.month()}
                                </button>
                            ))}
                        </>)
                    }}

                />
            )}
            {mode === 'time' && (
                <>
                    <div>
                        <button onClick={()=>{updateMode('data')}}>back</button>
                    </div>

                    <Radio.Group name="radiogroupSlots" onChange={handleTimeslot}>
                        {dates[selectDate.format('YYYY-MM-DD')].map( (el) => {
                            return (<Radio key={dayjs(el.startTime).format()} value={el}>{dayjs(el.startTime).format('hh-mm')} - {dayjs(el.endTime).format('hh-mm')}</Radio>)
                        })}
                    </Radio.Group>
                    <button onClick={handleSingupSession}>Записаться</button>
                </>
            )}
            {mode === 'pay' && (
                <ElementsForm amount={sessionCost}/>
            )}
        </Modal>
    );
};
